@@ -10,7 +10,6 @@ import inspect
 import logging
 import pathlib
 from types import FrameType
-import typing
 from typing import TYPE_CHECKING, Any, Callable
 
 import attrs
@@ -89,21 +88,21 @@ class CustomDeviceV2(BaseCustomDevice):
         ] = collections.defaultdict(list)
 
         # endpoints need to be modified before clusters
-        for add_endpoint_meta in quirk_metadata.adds_endpoint_metadata:
-            add_endpoint_meta(self)
-
         for remove_endpoint_meta in quirk_metadata.removes_endpoint_metadata:
             remove_endpoint_meta(self)
 
+        for add_endpoint_meta in quirk_metadata.adds_endpoint_metadata:
+            add_endpoint_meta(self)
+
         for replace_endpoint_meta in quirk_metadata.replaces_endpoint_metadata:
             replace_endpoint_meta(self)
 
-        for add_meta in quirk_metadata.adds_metadata:
-            add_meta(self)
-
         for remove_meta in quirk_metadata.removes_metadata:
             remove_meta(self)
 
+        for add_meta in quirk_metadata.adds_metadata:
+            add_meta(self)
+
         for replace_meta in quirk_metadata.replaces_metadata:
             replace_meta(self)
 
@@ -166,7 +165,7 @@ class AddsMetadata:
     cluster: int | type[Cluster | CustomCluster] = attrs.field()
     endpoint_id: int = attrs.field(default=1)
     cluster_type: ClusterType = attrs.field(default=ClusterType.Server)
-    constant_attributes: frozendict[ZCLAttributeDef, typing.Any] = attrs.field(
+    constant_attributes: frozendict[ZCLAttributeDef, Any] = attrs.field(
         factory=frozendict, converter=deepfreeze
     )
 
@@ -285,13 +284,18 @@ class RemovesEndpointMetadata:
 class ReplacesEndpointMetadata:
     """Replaces metadata for replacing an endpoint on a device."""
 
-    remove: RemovesEndpointMetadata = attrs.field()
-    add: AddsEndpointMetadata = attrs.field()
+    endpoint_id: int = attrs.field()
+    profile_id: int = attrs.field()
+    device_type: int = attrs.field()
 
     def __call__(self, device: CustomDeviceV2) -> None:
         """Process the replace."""
-        self.remove(device)
-        self.add(device)
+        if self.endpoint_id in device.endpoints:
+            ep: Endpoint = device.endpoints[self.endpoint_id]
+        else:
+            ep = device.add_endpoint(self.endpoint_id)
+        ep.profile_id = self.profile_id
+        ep.device_type = self.device_type
 
 
 @attrs.define(frozen=True, kw_only=True, repr=True)
@@ -344,10 +348,11 @@ class ZCLSensorMetadata(EntityMetadata):
     """Metadata for exposed ZCL attribute based sensor entity."""
 
     attribute_name: str | None = attrs.field(default=None)
-    attribute_converter: typing.Callable[[Any], Any] | None = attrs.field(default=None)
+    attribute_converter: Callable[[Any], Any] | None = attrs.field(default=None)
     reporting_config: ReportingConfig | None = attrs.field(default=None)
     divisor: int | None = attrs.field(default=None)
     multiplier: int | None = attrs.field(default=None)
+    suggested_display_precision: int | None = attrs.field(default=None)
     unit: str | None = attrs.field(default=None)
     device_class: SensorDeviceClass | None = attrs.field(default=None)
     state_class: SensorStateClass | None = attrs.field(default=None)
@@ -385,7 +390,7 @@ class BinarySensorMetadata(EntityMetadata):
     """Metadata for exposed binary sensor entity."""
 
     attribute_name: str = attrs.field()
-    attribute_converter: typing.Callable[[Any], Any] | None = attrs.field(default=None)
+    attribute_converter: Callable[[Any], Any] | None = attrs.field(default=None)
     reporting_config: ReportingConfig | None = attrs.field(default=None)
     device_class: BinarySensorDeviceClass | None = attrs.field(default=None)
 
@@ -447,7 +452,7 @@ class PreventDefaultEntityCreationMetadata:
     cluster_id: int | None = attrs.field()
     cluster_type: ClusterType | None = attrs.field()
     unique_id_suffix: str | None = attrs.field()
-    function: Callable[Any, bool] | None = attrs.field()
+    function: Callable[[Any], bool] | None = attrs.field()
 
 
 @attrs.define(frozen=True, kw_only=True, repr=True)
@@ -628,7 +633,7 @@ class QuirkBuilder:
         cluster: int | type[Cluster | CustomCluster],
         cluster_type: ClusterType = ClusterType.Server,
         endpoint_id: int = 1,
-        constant_attributes: dict[ZCLAttributeDef, typing.Any] | None = None,
+        constant_attributes: dict[ZCLAttributeDef, Any] | None = None,
     ) -> QuirkBuilder:
         """Add an AddsMetadata entry and returns self.
 
@@ -761,11 +766,9 @@ class QuirkBuilder:
         device_type: int = 0xFF,
     ) -> QuirkBuilder:
         """Add a ReplacesEndpointMetadata entry and return self."""
-        remove = RemovesEndpointMetadata(endpoint_id=endpoint_id)
-        add = AddsEndpointMetadata(
+        replace = ReplacesEndpointMetadata(
             endpoint_id=endpoint_id, profile_id=profile_id, device_type=device_type
         )
-        replace = ReplacesEndpointMetadata(remove=remove, add=add)
         self.replaces_endpoint_metadata.append(replace)
         return self
 
@@ -818,13 +821,14 @@ class QuirkBuilder:
         endpoint_id: int = 1,
         divisor: int = 1,
         multiplier: int = 1,
+        suggested_display_precision: int = 1,
         entity_type: EntityType = EntityType.STANDARD,
         device_class: SensorDeviceClass | None = None,
         state_class: SensorStateClass | None = None,
         unit: str | None = None,
         initially_disabled: bool = False,
         attribute_initialized_from_cache: bool = True,
-        attribute_converter: typing.Callable[[Any], Any] | None = None,
+        attribute_converter: Callable[[Any], Any] | None = None,
         reporting_config: ReportingConfig | None = None,
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
@@ -852,6 +856,7 @@ class QuirkBuilder:
                 attribute_converter=attribute_converter,
                 divisor=divisor,
                 multiplier=multiplier,
+                suggested_display_precision=suggested_display_precision,
                 unit=unit,
                 device_class=device_class,
                 state_class=state_class,
@@ -969,7 +974,7 @@ class QuirkBuilder:
         device_class: BinarySensorDeviceClass | None = None,
         initially_disabled: bool = False,
         attribute_initialized_from_cache: bool = True,
-        attribute_converter: typing.Callable[[Any], Any] | None = None,
+        attribute_converter: Callable[[Any], Any] | None = None,
         reporting_config: ReportingConfig | None = None,
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
@@ -1105,7 +1110,7 @@ class QuirkBuilder:
         cluster_id: int | None = None,
         cluster_type: ClusterType | None = None,
         unique_id_suffix: str | None = None,
-        function: Callable[Any, bool] | None = None,
+        function: Callable[[Any], bool] | None = None,
     ) -> QuirkBuilder:
         """Do not create default entities."""
         if cluster_id is not None and cluster_type is None:
