@@ -1,9 +1,11 @@
 """Typing helpers for Zigpy."""
+
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Dict, Union
+import enum
+from typing import TYPE_CHECKING, Any, Union
 
-ConfigType = Dict[str, Any]
+ConfigType = dict[str, Any]
 
 # pylint: disable=invalid-name
 ClusterType = "Cluster"
@@ -16,6 +18,16 @@ EndpointType = "Endpoint"
 ZDOType = "ZDO"
 AddressingMode = "AddressingMode"
 
+
+class UndefinedType(enum.Enum):
+    """Singleton type for use with not set sentinel values."""
+
+    _singleton = 0
+
+
+UNDEFINED = UndefinedType._singleton  # noqa: SLF001
+
+
 if TYPE_CHECKING:
     import zigpy.application
     import zigpy.device
@@ -28,7 +40,7 @@ if TYPE_CHECKING:
     ClusterType = zigpy.zcl.Cluster
     ControllerApplicationType = zigpy.application.ControllerApplication
     CustomClusterType = zigpy.quirks.CustomCluster
-    CustomDeviceType = zigpy.quirks.CustomDevice
+    CustomDeviceType = zigpy.quirks.BaseCustomDevice
     CustomEndpointType = zigpy.quirks.CustomEndpoint
     DeviceType = zigpy.device.Device
     EndpointType = zigpy.endpoint.Endpoint
