@@ -9,6 +9,7 @@ from zigpy.zcl import Cluster
 from zigpy.zcl.foundation import (
     BaseAttributeDefs,
     BaseCommandDefs,
+    Direction,
     ZCLAttributeDef,
     ZCLCommandDef,
 )
@@ -67,7 +68,7 @@ class Pump(Cluster):
     OperationMode: Final = OperationMode
     PumpStatus: Final = PumpStatus
 
-    cluster_id: Final = 0x0200
+    cluster_id: Final[t.uint16_t] = 0x0200
     name: Final = "Pump Configuration and Control"
     ep_attribute: Final = "pump"
 
@@ -347,7 +348,7 @@ class Thermostat(Cluster):
     RunningMode: Final = RunningMode
     RunningState: Final = RunningState
 
-    cluster_id: Final = 0x0201
+    cluster_id: Final[t.uint16_t] = 0x0201
     ep_attribute: Final = "thermostat"
 
     class AttributeDefs(BaseAttributeDefs):
@@ -498,7 +499,9 @@ class Thermostat(Cluster):
 
     class ServerCommandDefs(BaseCommandDefs):
         setpoint_raise_lower: Final = ZCLCommandDef(
-            id=0x00, schema={"mode": SetpointMode, "amount": t.int8s}, direction=False
+            id=0x00,
+            schema={"mode": SetpointMode, "amount": t.int8s},
+            direction=Direction.Client_to_Server,
         )
         set_weekly_schedule: Final = ZCLCommandDef(
             id=0x01,
@@ -508,17 +511,19 @@ class Thermostat(Cluster):
                 "mode_for_sequence": SeqMode,
                 "values": t.List[t.int16s],
             },
-            direction=False,
+            direction=Direction.Client_to_Server,
         )
         get_weekly_schedule: Final = ZCLCommandDef(
             id=0x02,
             schema={"days_to_return": SeqDayOfWeek, "mode_to_return": SeqMode},
-            direction=False,
+            direction=Direction.Client_to_Server,
         )
         clear_weekly_schedule: Final = ZCLCommandDef(
-            id=0x03, schema={}, direction=False
+            id=0x03, schema={}, direction=Direction.Client_to_Server
+        )
+        get_relay_status_log: Final = ZCLCommandDef(
+            id=0x04, schema={}, direction=Direction.Client_to_Server
         )
-        get_relay_status_log: Final = ZCLCommandDef(id=0x04, schema={}, direction=False)
 
     class ClientCommandDefs(BaseCommandDefs):
         get_weekly_schedule_response: Final = ZCLCommandDef(
@@ -529,7 +534,7 @@ class Thermostat(Cluster):
                 "mode_for_sequence": SeqMode,
                 "values": t.List[t.int16s],
             },
-            direction=True,
+            direction=Direction.Server_to_Client,
         )
         get_relay_status_log_response: Final = ZCLCommandDef(
             id=0x01,
@@ -541,7 +546,7 @@ class Thermostat(Cluster):
                 "set_point": t.int16s,
                 "unread_entries": t.uint16_t,
             },
-            direction=True,
+            direction=Direction.Server_to_Client,
         )
 
 
@@ -571,7 +576,7 @@ class Fan(Cluster):
     FanMode: Final = FanMode
     FanModeSequence: Final = FanModeSequence
 
-    cluster_id: Final = 0x0202
+    cluster_id: Final[t.uint16_t] = 0x0202
     name: Final = "Fan Control"
     ep_attribute: Final = "fan"
 
@@ -604,7 +609,7 @@ class Dehumidification(Cluster):
     DehumidificationLockout: Final = DehumidificationLockout
     RelativeHumidityDisplay: Final = RelativeHumidityDisplay
 
-    cluster_id: Final = 0x0203
+    cluster_id: Final[t.uint16_t] = 0x0203
     ep_attribute: Final = "dehumidification"
 
     class AttributeDefs(BaseAttributeDefs):
@@ -665,7 +670,7 @@ class UserInterface(Cluster):
     KeypadLockout: Final = KeypadLockout
     ScheduleProgrammingVisibility: Final = ScheduleProgrammingVisibility
 
-    cluster_id: Final = 0x0204
+    cluster_id: Final[t.uint16_t] = 0x0204
     name: Final = "Thermostat User Interface Configuration"
     ep_attribute: Final = "thermostat_ui"
 
