@@ -153,7 +153,9 @@ class IasZone(Cluster):
             and self.is_server
             and not hdr.frame_control.disable_default_response
         ):
-            hdr.frame_control.is_reply = False  # this is a client -> server cmd
+            hdr.frame_control = hdr.frame_control.replace(
+                direction=Direction.Client_to_Server
+            )  # this is a client -> server cmd
             self.send_default_rsp(hdr, foundation.Status.SUCCESS)
 
 
@@ -494,8 +496,8 @@ class Squawk(_SquawkOrWarningCommand):
 
 class IasWd(Cluster):
     """The IAS WD cluster provides an interface to the functionality of any Warning
-    Device equipment of the IAS system. Using this cluster, a ZigBee enabled CIE device
-    can access a ZigBee enabled IAS WD device and issue alarm warning indications
+    Device equipment of the IAS system. Using this cluster, a Zigbee enabled CIE device
+    can access a Zigbee enabled IAS WD device and issue alarm warning indications
     (siren, strobe lighting, etc.) when a system alarm condition is detected
     """
 
