@@ -7,18 +7,20 @@ from zigpy.zcl import Cluster
 from zigpy.zcl.foundation import (
     BaseAttributeDefs,
     BaseCommandDefs,
+    DataType,
+    Direction,
     ZCLAttributeDef,
     ZCLCommandDef,
 )
 
 
 class Price(Cluster):
-    cluster_id: Final = 0x0700
+    cluster_id: Final[t.uint16_t] = 0x0700
     ep_attribute: Final = "smartenergy_price"
 
 
 class Drlc(Cluster):
-    cluster_id: Final = 0x0701
+    cluster_id: Final[t.uint16_t] = 0x0701
     ep_attribute: Final = "smartenergy_drlc"
 
 
@@ -41,10 +43,106 @@ class RegisteredTier(t.enum8):
     Extended_Tier = 0x0F
 
 
+class MeteringDeviceType(t.enum8):
+    """Metering device type."""
+
+    Electric_Metering = 0
+    Gas_Metering = 1
+    Water_Metering = 2
+    Thermal_Metering = 3  # Deprecated
+    Pressure_Metering = 4
+    Heat_Metering = 5
+    Cooling_Metering = 6
+    EUMD_for_metering_electric_vehicle_charging = 7
+    PV_Generation_Metering = 8
+    Wind_Turbine_Generation_Metering = 9
+    Water_Turbine_Generation_Metering = 10
+    Micro_Generation_Metering = 11
+    Solar_Hot_Water_Generation_Metering = 12
+    Electric_Metering_Element_Phase_1 = 13
+    Electric_Metering_Element_Phase_2 = 14
+    Electric_Metering_Element_Phase_3 = 15
+
+    # 127 + above enum values
+    Mirrored_Electric_Metering = 127
+    Mirrored_Gas_Metering = 128
+    Mirrored_Water_Metering = 129
+    Mirrored_Thermal_Metering = 130  # Deprecated
+    Mirrored_Pressure_Metering = 131
+    Mirrored_Heat_Metering = 132
+    Mirrored_Cooling_Metering = 133
+    Mirrored_EUMD_for_metering_electric_vehicle_charging = 134
+    Mirrored_PV_Generation_Metering = 135
+    Mirrored_Wind_Turbine_Generation_Metering = 136
+    Mirrored_Water_Turbine_Generation_Metering = 137
+    Mirrored_Micro_Generation_Metering = 138
+    Mirrored_Solar_Hot_Water_Generation_Metering = 139
+    Mirrored_Electric_Metering_Element_Phase_1 = 140
+    Mirrored_Electric_Metering_Element_Phase_2 = 141
+    Mirrored_Electric_Metering_Element_Phase_3 = 142
+
+
+class MeteringUnitofMeasure(t.enum8):
+    """Metering unit of measure."""
+
+    Kwh_and_Kwh_binary = 0x00
+    Cubic_Meter_and_Cubic_Meter_per_Hour_binary = 0x01
+    Cubic_Feet_and_Cubic_Feet_per_Hour_binary = 0x02
+    Ccf_and_Ccf_per_Hour_binary = 0x03
+    US_Gallons_and_US_Gallons_per_Hour_binary = 0x04
+    Imperial_Gallons_and_Imperial_Gallons_per_Hour_binary = 0x05
+    BTU_and_BTU_per_Hour_binary = 0x06
+    Liters_and_Liters_per_Hour_binary = 0x07
+    KPA_gauge_binary = 0x08
+    KPA_absolute_binary = 0x09
+    MCF_and_MCF_per_Hour_binary = 0x0A
+    Unitless_binary = 0x0B
+    Mega_Joule_and_Mega_Joule_per_second_binary = 0x0C
+    Kvar_and_Kvarh_binary = 0x0D
+    Kwh_and_Kwh_bcd = 0x80
+    Cubic_Meter_and_Cubic_Meter_per_Hour_bcd = 0x81
+    Cubic_Feet_and_Cubic_Feet_per_Hour_bcd = 0x82
+    Ccf_and_Ccf_per_Hour_bcd = 0x83
+    US_Gallons_and_US_Gallons_per_Hour_bcd = 0x84
+    Imperial_Gallons_and_Imperial_Gallons_per_Hour_bcd = 0x85
+    BTU_and_BTU_per_Hour_bcd = 0x86
+    Liters_and_Liters_per_Hour_bcd = 0x87
+    KPA_gauge_bcd = 0x88
+    KPA_absolute_bcd = 0x89
+    MCF_and_MCF_per_Hour_bcd = 0x8A
+    Unitless_bcd = 0x8B
+    Mega_Joule_and_Mega_Joule_per_second_bcd = 0x8C
+    Kvar_and_Kvarh_bcd = 0x8D
+
+
+class NumberFormatting(t.IntStruct, t.uint8_t):
+    """Number formatting."""
+
+    num_digits_right_of_decimal: t.uint3_t
+    num_digits_left_of_decimal: t.uint4_t
+    suppress_leading_zeros: t.uint1_t
+
+
+class MeteringStatus(t.bitmap8):
+    """Metering status."""
+
+    Check_Meter = 0b00000001
+    Low_Battery = 0b00000010
+    Tamper_Detect = 0b00000100
+    Power_Failure = 0b00001000
+    Power_Quality = 0b00010000
+    Leak_Detect = 0b00100000
+    Service_Disconnect_Open = 0b01000000
+    Reserved = 0b10000000
+
+
 class Metering(Cluster):
     RegisteredTier: Final = RegisteredTier
+    MeteringDeviceType: Final = MeteringDeviceType
+    MeteringUnitofMeasure: Final = MeteringUnitofMeasure
+    NumberFormatting: Final = NumberFormatting
 
-    cluster_id: Final = 0x0702
+    cluster_id: Final[t.uint16_t] = 0x0702
     ep_attribute: Final = "smartenergy_metering"
 
     class AttributeDefs(BaseAttributeDefs):
@@ -80,10 +178,10 @@ class Metering(Cluster):
         fast_poll_update_period: Final = ZCLAttributeDef(
             id=0x000B, type=t.uint8_t, access="r"
         )
-        current_block_period_consump_delivered: Final = ZCLAttributeDef(
+        current_block_period_consumption_delivered: Final = ZCLAttributeDef(
             id=0x000C, type=t.uint48_t, access="r"
         )
-        daily_consump_target: Final = ZCLAttributeDef(
+        daily_consumption_target: Final = ZCLAttributeDef(
             id=0x000D, type=t.uint24_t, access="r"
         )
         current_block: Final = ZCLAttributeDef(id=0x000E, type=t.enum8, access="r")
@@ -118,7 +216,7 @@ class Metering(Cluster):
         current_out_energy_carrier_demand: Final = ZCLAttributeDef(
             id=0x001B, type=t.int24s, access="r"
         )
-        current_block_period_consump_received: Final = ZCLAttributeDef(
+        current_block_period_consumption_received: Final = ZCLAttributeDef(
             id=0x001D, type=t.uint48_t, access="r"
         )
         current_block_received: Final = ZCLAttributeDef(
@@ -227,7 +325,7 @@ class Metering(Cluster):
         current_tier15_summ_received: Final = ZCLAttributeDef(
             id=0x011D, type=t.uint48_t, access="r"
         )
-        status: Final = ZCLAttributeDef(id=0x0200, type=t.bitmap8, access="r")
+        status: Final = ZCLAttributeDef(id=0x0200, type=MeteringStatus, access="r")
         remaining_battery_life: Final = ZCLAttributeDef(
             id=0x0201, type=t.uint8_t, access="r"
         )
@@ -243,20 +341,63 @@ class Metering(Cluster):
         iambient_consumption_indicator: Final = ZCLAttributeDef(
             id=0x0207, type=t.enum8, access="r"
         )
-        unit_of_measure: Final = ZCLAttributeDef(id=0x0300, type=t.enum8, access="r")
+        unit_of_measure: Final = ZCLAttributeDef(
+            id=0x0300, type=MeteringUnitofMeasure, access="r"
+        )
         multiplier: Final = ZCLAttributeDef(id=0x0301, type=t.uint24_t, access="r")
         divisor: Final = ZCLAttributeDef(id=0x0302, type=t.uint24_t, access="r")
+
+        # This attribute shall be used against the following attributes:
+        # • CurrentSummationDelivered
+        # • CurrentSummationReceived
+        # • SummationDeliveredPerReport
+        # • TOU Information attributes
+        # • DFTSummation
+        # • Block Information attributes
         summation_formatting: Final = ZCLAttributeDef(
-            id=0x0303, type=t.bitmap8, access="r"
+            id=0x0303, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
+
+        # This attribute shall be used against the following attributes:
+        # • CurrentMaxDemandDelivered
+        # • CurrentMaxDemandReceived
+        # • InstantaneousDemand
         demand_formatting: Final = ZCLAttributeDef(
-            id=0x0304, type=t.bitmap8, access="r"
+            id=0x0304, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
-        historical_consump_formatting: Final = ZCLAttributeDef(
-            id=0x0305, type=t.bitmap8, access="r"
+
+        # This attribute shall be used against the following attributes:
+        # • CurrentDayConsumptionDelivered
+        # • CurrentDayConsumptionReceived
+        # • PreviousDayConsumptionDelivered
+        # • PreviousDayConsumptionReceived
+        # • CurrentPartialProfileIntervalValue
+        # • Intervals
+        # • DailyConsumptionTarget
+        # • CurrentDayConsumptionDelivered
+        # • CurrentDayConsumptionReceived
+        # • PreviousDayNConsumptionDelivered
+        # • PreviousDayNConsumptionReceived
+        # • CurrentWeekConsumptionDelivered
+        # • CurrentWeekConsumptionReceived
+        # • PreviousWeekNConsumptionDelivered
+        # • PreviousWeekNConsumptionReceived
+        # • CurrentMonthConsumptionDelivered
+        # • CurrentMonthConsumptionReceived
+        # • PreviousMonthNConsumptionDelivered
+        # • PreviousMonthNConsumptionReceived
+        historical_consumption_formatting: Final = ZCLAttributeDef(
+            id=0x0305, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
         metering_device_type: Final = ZCLAttributeDef(
-            id=0x0306, type=t.bitmap8, access="r"
+            id=0x0306,
+            type=MeteringDeviceType,
+            # Note that these values represent an Enumeration, and not an 8-bit bitmap
+            # as indicated in the attribute description. For backwards compatibility
+            # reasons, the data type has not been changed, though the data itself should
+            # be treated like an enum
+            zcl_type=DataType.map8,
+            access="r",
         )
         site_id: Final = ZCLAttributeDef(
             id=0x0307, type=t.LimitedLVBytes(32), access="r"
@@ -264,20 +405,25 @@ class Metering(Cluster):
         meter_serial_number: Final = ZCLAttributeDef(
             id=0x0308, type=t.LimitedLVBytes(24), access="r"
         )
-        energy_carrier_unit_of_meas: Final = ZCLAttributeDef(
-            id=0x0309, type=t.enum8, access="r"
+        energy_carrier_unit_of_measure: Final = ZCLAttributeDef(
+            id=0x0309, type=MeteringUnitofMeasure, access="r"
         )
-        energy_carrier_summ_formatting: Final = ZCLAttributeDef(
-            id=0x030A, type=t.bitmap8, access="r"
+        energy_carrier_summation_formatting: Final = ZCLAttributeDef(
+            id=0x030A, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
         energy_carrier_demand_formatting: Final = ZCLAttributeDef(
-            id=0x030B, type=t.bitmap8, access="r"
+            id=0x030B, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
         temperature_unit_of_measure: Final = ZCLAttributeDef(
-            id=0x030C, type=t.enum8, access="r"
+            id=0x030C, type=MeteringUnitofMeasure, access="r"
         )
+
+        # This attribute shall be used in relation with the following attributes:
+        # • InletTemperature
+        # • OutletTemperature
+        # • ControlTemperature
         temperature_formatting: Final = ZCLAttributeDef(
-            id=0x030D, type=t.bitmap8, access="r"
+            id=0x030D, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
         module_serial_number: Final = ZCLAttributeDef(
             id=0x030E, type=t.LimitedLVBytes(24), access="r"
@@ -292,27 +438,46 @@ class Metering(Cluster):
             id=0x0311, type=t.LimitedLVBytes(24), access="r"
         )
         alternative_unit_of_measure: Final = ZCLAttributeDef(
-            id=0x0312, type=t.enum8, access="r"
+            id=0x0312, type=MeteringUnitofMeasure, access="r"
         )
+
+        # This attribute shall be used against the following attribute:
+        # • AlternativeInstantaneousDemand
         alternative_demand_formatting: Final = ZCLAttributeDef(
-            id=0x0313, type=t.bitmap8, access="r"
+            id=0x0313, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
+        # This attribute shall be used against the following attributes:
+        # • CurrentDayAlternativeConsumptionDelivered
+        # • CurrentDayAlternativeConsumptionReceived
+        # • PreviousDayAlternativeConsumptionDelivered
+        # • PreviousDayAlternativeConsumptionReceived
+        # • CurrentAlternativePartialProfileIntervalValue
+        # • PreviousDayNAlternativeConsumptionDelivered
+        # • PreviousDayNAlternativeConsumptionReceived
+        # • CurrentWeekAlternativeConsumptionDelivered
+        # • CurrentWeekAlternativeConsumptionReceived
+        # • PreviousWeekNAlternativeConsumptionDelivered
+        # • PreviousWeekNAlternativeConsumptionReceived
+        # • CurrentMonthAlternativeConsumptionDelivered
+        # • CurrentMonthAlternativeConsumptionReceived
+        # • PreviousMonthNAlternativeConsumptionDelivered
+        # • PreviousMonthNAlternativeConsumptionReceived
         alternative_consumption_formatting: Final = ZCLAttributeDef(
-            id=0x0314, type=t.bitmap8, access="r"
+            id=0x0314, zcl_type=DataType.map8, type=NumberFormatting, access="r"
         )
         instantaneous_demand: Final = ZCLAttributeDef(
             id=0x0400, type=t.int24s, access="r"
         )
-        currentday_consump_delivered: Final = ZCLAttributeDef(
+        currentday_consumption_delivered: Final = ZCLAttributeDef(
             id=0x0401, type=t.uint24_t, access="r"
         )
-        currentday_consump_received: Final = ZCLAttributeDef(
+        currentday_consumption_received: Final = ZCLAttributeDef(
             id=0x0402, type=t.uint24_t, access="r"
         )
-        previousday_consump_delivered: Final = ZCLAttributeDef(
+        previousday_consumption_delivered: Final = ZCLAttributeDef(
             id=0x0403, type=t.uint24_t, access="r"
         )
-        previousday_consump_received: Final = ZCLAttributeDef(
+        previousday_consumption_received: Final = ZCLAttributeDef(
             id=0x0404, type=t.uint24_t, access="r"
         )
         cur_part_profile_int_start_time_delivered: Final = ZCLAttributeDef(
@@ -419,66 +584,86 @@ class Metering(Cluster):
         )
 
     class ServerCommandDefs(BaseCommandDefs):
-        get_profile: Final = ZCLCommandDef(id=0x00, schema={}, direction=False)
-        req_mirror: Final = ZCLCommandDef(id=0x01, schema={}, direction=False)
-        mirror_rem: Final = ZCLCommandDef(id=0x02, schema={}, direction=False)
-        req_fast_poll_mode: Final = ZCLCommandDef(id=0x03, schema={}, direction=False)
-        get_snapshot: Final = ZCLCommandDef(id=0x04, schema={}, direction=False)
-        take_snapshot: Final = ZCLCommandDef(id=0x05, schema={}, direction=False)
+        get_profile: Final = ZCLCommandDef(
+            id=0x00, schema={}, direction=Direction.Client_to_Server
+        )
+        req_mirror: Final = ZCLCommandDef(
+            id=0x01, schema={}, direction=Direction.Client_to_Server
+        )
+        mirror_rem: Final = ZCLCommandDef(
+            id=0x02, schema={}, direction=Direction.Client_to_Server
+        )
+        req_fast_poll_mode: Final = ZCLCommandDef(
+            id=0x03, schema={}, direction=Direction.Client_to_Server
+        )
+        get_snapshot: Final = ZCLCommandDef(
+            id=0x04, schema={}, direction=Direction.Client_to_Server
+        )
+        take_snapshot: Final = ZCLCommandDef(
+            id=0x05, schema={}, direction=Direction.Client_to_Server
+        )
         mirror_report_attr_response: Final = ZCLCommandDef(
-            id=0x06, schema={}, direction=True
+            id=0x06, schema={}, direction=Direction.Server_to_Client
         )
 
     class ClientCommandDefs(BaseCommandDefs):
-        get_profile_response: Final = ZCLCommandDef(id=0x00, schema={}, direction=True)
-        req_mirror_response: Final = ZCLCommandDef(id=0x01, schema={}, direction=True)
-        mirror_rem_response: Final = ZCLCommandDef(id=0x02, schema={}, direction=True)
+        get_profile_response: Final = ZCLCommandDef(
+            id=0x00, schema={}, direction=Direction.Server_to_Client
+        )
+        req_mirror_response: Final = ZCLCommandDef(
+            id=0x01, schema={}, direction=Direction.Server_to_Client
+        )
+        mirror_rem_response: Final = ZCLCommandDef(
+            id=0x02, schema={}, direction=Direction.Server_to_Client
+        )
         req_fast_poll_mode_response: Final = ZCLCommandDef(
-            id=0x03, schema={}, direction=True
+            id=0x03, schema={}, direction=Direction.Server_to_Client
+        )
+        get_snapshot_response: Final = ZCLCommandDef(
+            id=0x04, schema={}, direction=Direction.Server_to_Client
         )
-        get_snapshot_response: Final = ZCLCommandDef(id=0x04, schema={}, direction=True)
 
 
 class Messaging(Cluster):
-    cluster_id: Final = 0x0703
+    cluster_id: Final[t.uint16_t] = 0x0703
     ep_attribute: Final = "smartenergy_messaging"
 
 
 class Tunneling(Cluster):
-    cluster_id: Final = 0x0704
+    cluster_id: Final[t.uint16_t] = 0x0704
     ep_attribute: Final = "smartenergy_tunneling"
 
 
 class Prepayment(Cluster):
-    cluster_id: Final = 0x0705
+    cluster_id: Final[t.uint16_t] = 0x0705
     ep_attribute: Final = "smartenergy_prepayment"
 
 
 class EnergyManagement(Cluster):
-    cluster_id: Final = 0x0706
+    cluster_id: Final[t.uint16_t] = 0x0706
     ep_attribute: Final = "smartenergy_energy_management"
 
 
 class Calendar(Cluster):
-    cluster_id: Final = 0x0707
+    cluster_id: Final[t.uint16_t] = 0x0707
     ep_attribute: Final = "smartenergy_calendar"
 
 
 class DeviceManagement(Cluster):
-    cluster_id: Final = 0x0708
+    cluster_id: Final[t.uint16_t] = 0x0708
     ep_attribute: Final = "smartenergy_device_management"
 
 
 class Events(Cluster):
-    cluster_id: Final = 0x0709
+    cluster_id: Final[t.uint16_t] = 0x0709
     ep_attribute: Final = "smartenergy_events"
 
 
 class MduPairing(Cluster):
-    cluster_id: Final = 0x070A
+    cluster_id: Final[t.uint16_t] = 0x070A
     ep_attribute: Final = "smartenergy_mdu_pairing"
 
 
 class KeyEstablishment(Cluster):
-    cluster_id: Final = 0x0800
+    cluster_id: Final[t.uint16_t] = 0x0800
     ep_attribute: Final = "smartenergy_key_establishment"
